#ifndef lint
static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
#endif

#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYPATCH 20140101

#define YYEMPTY        (-1)
#define yyclearin      (yychar = YYEMPTY)
#define yyerrok        (yyerrflag = 0)
#define YYRECOVERING() (yyerrflag != 0)


#ifndef yyparse
#define yyparse    cue_scanner_parse
#endif /* yyparse */

#ifndef yylex
#define yylex      cue_scanner_lex
#endif /* yylex */

#ifndef yyerror
#define yyerror    cue_scanner_error
#endif /* yyerror */

#ifndef yychar
#define yychar     cue_scanner_char
#endif /* yychar */

#ifndef yyval
#define yyval      cue_scanner_val
#endif /* yyval */

#ifndef yylval
#define yylval     cue_scanner_lval
#endif /* yylval */

#ifndef yydebug
#define yydebug    cue_scanner_debug
#endif /* yydebug */

#ifndef yynerrs
#define yynerrs    cue_scanner_nerrs
#endif /* yynerrs */

#ifndef yyerrflag
#define yyerrflag  cue_scanner_errflag
#endif /* yyerrflag */

#ifndef yylhs
#define yylhs      cue_scanner_lhs
#endif /* yylhs */

#ifndef yylen
#define yylen      cue_scanner_len
#endif /* yylen */

#ifndef yydefred
#define yydefred   cue_scanner_defred
#endif /* yydefred */

#ifndef yydgoto
#define yydgoto    cue_scanner_dgoto
#endif /* yydgoto */

#ifndef yysindex
#define yysindex   cue_scanner_sindex
#endif /* yysindex */

#ifndef yyrindex
#define yyrindex   cue_scanner_rindex
#endif /* yyrindex */

#ifndef yygindex
#define yygindex   cue_scanner_gindex
#endif /* yygindex */

#ifndef yytable
#define yytable    cue_scanner_table
#endif /* yytable */

#ifndef yycheck
#define yycheck    cue_scanner_check
#endif /* yycheck */

#ifndef yyname
#define yyname     cue_scanner_name
#endif /* yyname */

#ifndef yyrule
#define yyrule     cue_scanner_rule
#endif /* yyrule */
#define YYPREFIX "cue_scanner_"

#define YYPURE 0

/*
 * CUE parser functions
 *
 * Copyright (C) 2010-2017, Joachim Metz <joachim.metz@gmail.com>
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <common.h>
#include <narrow_string.h>
#include <types.h>

#if defined( HAVE_STDLIB_H ) || defined( WINAPI )
#include <stdlib.h>
#endif

#include "libodraw_definitions.h"
#include "libodraw_handle.h"
#include "libodraw_libcerror.h"
#include "libodraw_libcnotify.h"
#include "libodraw_metadata.h"
#include "libodraw_types.h"

#define YYMALLOC	cue_scanner_alloc
#define YYREALLOC	cue_scanner_realloc
#define YYFREE		cue_scanner_free

#define YYLEX_PARAM	NULL
#define YYPARSE_PARAM	parser_state

#if defined( HAVE_DEBUG_OUTPUT )
#define cue_parser_rule_print( string ) \
	if( libcnotify_verbose != 0 ) libcnotify_printf( "cue_parser: rule: %s\n", string )
#else
#define cue_parser_rule_print( string )
#endif

#if !defined( CD_SECS )
/* seconds per minute */
#define CD_SECS			60
#endif

#if !defined( CD_FRAMES )
/* frames per second */
#define CD_FRAMES		75
#endif

/* Note that the MSF are relative, so there is no need for the MSF offset correction
 */
#define cue_parser_copy_relative_msf_to_lba( msf_string, lba ) \
        lba  = ( ( msf_string[ 0 ] - '0' ) * 10 ) + ( msf_string[ 1 ] - '0' ); \
        lba *= CD_SECS; \
        lba += ( ( msf_string[ 3 ] - '0' ) * 10 ) + ( msf_string[ 4 ] - '0' ); \
        lba *= CD_FRAMES; \
        lba += ( ( msf_string[ 6 ] - '0' ) * 10 ) + ( msf_string[ 7 ] - '0' );
#ifdef YYSTYPE
#undef  YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#endif
#ifndef YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
typedef union
{
        /* The numeric value
         */
        uint32_t numeric_value;

        /* The string value
         */
	struct cue_string_value
	{
		/* The string data
		 */
	        const char *data;

		/* The string length
		 */
		size_t length;

	} string_value;
} YYSTYPE;
#endif /* !YYSTYPE_IS_DECLARED */

typedef struct cue_parser_state cue_parser_state_t;

struct cue_parser_state
{
	/* The file
	 */
	libodraw_handle_t *handle;

	/* The error
	 */
	libcerror_error_t **error;

	/* The previous file index
	 */
	int previous_file_index;

	/* The current file index
	 */
	int current_file_index;

	/* The file type
	 */
	uint8_t file_type;

	/* Value to indicate a new file command was issued
	 */
	uint8_t new_file;

	/* The previous file sector
	 */
	uint64_t previous_file_sector;

	/* The file sector
	 */
	uint64_t file_sector;

	/* The previous session start sector
	 */
	uint64_t previous_session_start_sector;

	/* The previous lead-out start sector
	 */
	uint64_t previous_lead_out_start_sector;

	/* The previous track start sector
	 */
	uint64_t previous_track_start_sector;

	/* The current start sector
	 */
	uint64_t current_start_sector;

	/* The session number of sectors
	 */
	uint64_t session_number_of_sectors;

	/* The lead-out number of sectors
	 */
	uint64_t lead_out_number_of_sectors;

	/* The track number of sectors
	 */
	uint64_t track_number_of_sectors;

	/* The previous track type
	 */
	uint8_t previous_track_type;

	/* The current track type
	 */
	uint8_t current_track_type;

	/* The previous session
	 */
	int previous_session;

	/* The current session
	 */
	int current_session;

	/* The previous lead-out
	 */
	int previous_lead_out;

	/* The current lead-out
	 */
	int current_lead_out;

	/* The previous track
	 */
	int previous_track;

	/* The current track
	 */
	int current_track;

	/* The previous index
	 */
	int previous_index;

	/* The current index
	 */
	int current_index;
};

typedef size_t yy_size_t;
typedef struct yy_buffer_state* YY_BUFFER_STATE;

extern int cue_scanner_suppress_error;

extern int cue_scanner_lex_destroy(
            void );

extern void *cue_scanner_alloc(
              yy_size_t size );

extern void *cue_scanner_realloc(
              void *buffer,
              yy_size_t size );

extern void *cue_scanner_free(
              void *buffer );

extern int cue_scanner_lex(
            void *user_data );

extern void cue_scanner_error(
             void *parser_state,
             const char *error_string );

extern YY_BUFFER_STATE cue_scanner__scan_buffer(
                        char *buffer,
                        yy_size_t buffer_size );

extern void cue_scanner__delete_buffer(
             YY_BUFFER_STATE buffer_state );

extern size_t cue_scanner_buffer_offset;

static char *cue_parser_function = "cue_parser";

int cue_parser_parse_buffer(
     libodraw_handle_t *handle,
     const uint8_t *buffer,
     size_t buffer_size,
     libcerror_error_t **error );


/* compatibility with bison */
#ifdef YYPARSE_PARAM
/* compatibility with FreeBSD */
# ifdef YYPARSE_PARAM_TYPE
#  define YYPARSE_DECL() yyparse(YYPARSE_PARAM_TYPE YYPARSE_PARAM)
# else
#  define YYPARSE_DECL() yyparse(void *YYPARSE_PARAM)
# endif
#else
# define YYPARSE_DECL() yyparse(void * parser_state)
#endif

/* Parameters sent to lex. */
#ifdef YYLEX_PARAM
# define YYLEX_DECL() yylex(void *YYLEX_PARAM)
# define YYLEX yylex(YYLEX_PARAM)
#else
# define YYLEX_DECL() yylex((void *)  NULL)
# define YYLEX yylex(NULL)
#endif

/* Parameters sent to yyerror. */
#ifndef YYERROR_DECL
#define YYERROR_DECL() yyerror(void * parser_state, const char *s)
#endif
#ifndef YYERROR_CALL
#define YYERROR_CALL(msg) yyerror(parser_state, msg)
#endif

extern int YYPARSE_DECL();

#define CUE_END_OF_LINE 257
#define CUE_SEMI_COLON 258
#define CUE_2DIGIT 259
#define CUE_CATALOG_NUMBER 260
#define CUE_ISRC_CODE 261
#define CUE_KEYWORD_STRING 262
#define CUE_MSF 263
#define CUE_STRING 264
#define CUE_CATALOG 265
#define CUE_CDTEXTFILE 266
#define CUE_FLAGS 267
#define CUE_FILE 268
#define CUE_INDEX 269
#define CUE_ISRC 270
#define CUE_POSTGAP 271
#define CUE_PREGAP 272
#define CUE_REMARK 273
#define CUE_TRACK 274
#define CUE_CDTEXT_ARRANGER 275
#define CUE_CDTEXT_COMPOSER 276
#define CUE_CDTEXT_DISC_ID 277
#define CUE_CDTEXT_GENRE 278
#define CUE_CDTEXT_MESSAGE 279
#define CUE_CDTEXT_PERFORMER 280
#define CUE_CDTEXT_SIZE_INFO 281
#define CUE_CDTEXT_SONGWRITER 282
#define CUE_CDTEXT_TITLE 283
#define CUE_CDTEXT_TOC_INFO1 284
#define CUE_CDTEXT_TOC_INFO2 285
#define CUE_CDTEXT_UPC_EAN 286
#define CUE_REMARK_LEAD_OUT 287
#define CUE_REMARK_ORIGINAL_MEDIA_TYPE 288
#define CUE_REMARK_RUN_OUT 289
#define CUE_REMARK_SESSION 290
#define YYERRCODE 256
static const short cue_scanner_lhs[] = {                 -1,
    0,    1,    1,    4,    4,    4,    4,    4,    3,    3,
   10,   11,   11,   16,   16,   16,   13,   13,   17,   17,
   17,   17,   17,   17,   14,   14,   21,   21,   21,   15,
   15,   23,   23,   23,   23,    5,    7,   25,   25,   25,
   25,   25,   25,   25,   25,   25,   25,   25,   25,    6,
    2,   18,   26,   26,   22,   19,   24,   20,    8,    8,
    8,    8,    8,   27,   28,   29,   32,   30,   31,   12,
    9,
};
static const short cue_scanner_len[] = {                  2,
    4,    0,    2,    1,    1,    1,    1,    1,    0,    2,
    5,    0,    2,    1,    1,    1,    0,    2,    1,    1,
    1,    1,    1,    1,    0,    2,    1,    1,    1,    0,
    2,    1,    1,    1,    1,    3,    3,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    3,
    4,    3,    0,    2,    4,    3,    3,    3,    1,    1,
    1,    1,    1,    3,    3,    3,    1,    3,    3,    4,
    1,
};
static const short cue_scanner_defred[] = {               0,
   71,    0,    0,    0,   38,   39,   40,   41,   42,   43,
   44,   45,   46,   47,   48,   49,    0,    0,    0,    0,
    0,    0,    0,    4,    5,    6,    7,    8,    0,   59,
   60,   61,   62,   63,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    3,    0,    0,   36,   50,   64,   65,
   68,   69,    0,    0,   37,   66,    0,   14,    1,   15,
   16,    0,    0,    0,   51,   10,    0,    0,   13,    0,
    0,    0,    0,   19,   23,   24,    0,    0,   20,   21,
   22,    0,    0,    0,    0,    0,    0,   28,   29,    0,
    0,   27,   18,   70,   54,   52,   56,   58,    0,    0,
   32,   34,   35,   11,    0,   33,   26,    0,    0,   31,
   55,   57,
};
static const short cue_scanner_dgoto[] = {               21,
   22,   58,   59,   23,   24,   25,   26,   27,   28,   62,
   63,   68,   77,   90,  104,   64,   78,   79,   80,   81,
   91,   92,  105,  106,   29,   84,   30,   31,   32,   33,
   34,   35,
};
static const short cue_scanner_sindex[] = {            -182,
    0, -260, -261,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, -259, -257, -254, -253,
    0, -250, -182,    0,    0,    0,    0,    0, -237,    0,
    0,    0,    0,    0, -245, -243, -228, -225, -224, -223,
 -221, -227, -182,    0, -217, -215,    0,    0,    0,    0,
    0,    0, -214, -114,    0,    0, -212,    0,    0,    0,
    0, -114, -222, -114,    0,    0, -213, -216,    0, -207,
 -186, -184, -185,    0,    0,    0, -109, -216,    0,    0,
    0, -210, -186, -208, -178, -177, -174,    0,    0, -148,
 -109,    0,    0,    0,    0,    0,    0,    0, -181, -176,
    0,    0,    0,    0, -148,    0,    0, -171, -169,    0,
    0,    0,
};
static const short cue_scanner_rindex[] = {            -187,
    0,    0,    0, -167,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   22,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   22,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   19,    0,    0,    0,    0,    0,    0,
    0,   19,    0, -164,    0,    0,    0,   20,    0,    0,
 -165,    0,    0,    0,    0,    0,    1,   20,    0,    0,
    0,    0, -165,    0,    0,    0,    0,    0,    0,   24,
    1,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   24,    0,    0,    0,    0,    0,
    0,    0,
};
static const short cue_scanner_gindex[] = {               0,
  -15,   68,   49,    0,    0,    0,  -55,  -52,  -47,    0,
   48,    0,   35,   23,   10,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   33,    0,    0,    0,    0,
    0,    0,
};
#define YYTABLESIZE 298
static const short cue_scanner_table[] = {               36,
   25,   60,   37,   38,   39,   41,   61,   44,   40,   60,
   46,   60,   74,   47,   61,   75,   61,   42,    9,   17,
   76,    2,   74,   30,   88,   75,   45,   54,   48,   89,
   76,   49,   50,   51,  101,   52,   53,  102,   88,   55,
    1,   56,  103,   89,   65,   70,   94,   57,   96,  101,
   71,   67,  102,   72,   82,   73,    4,  103,    5,    6,
    7,    8,    9,   10,   11,   12,   13,   14,   15,   16,
   17,   18,   19,   20,    1,   83,   85,   86,   97,   98,
    2,  108,    2,    3,   99,  111,  109,  112,   67,   43,
    4,   53,    5,    6,    7,    8,    9,   10,   11,   12,
   13,   14,   15,   16,   17,   18,   19,   20,    1,   12,
   66,   69,   93,  107,  110,   95,    0,    0,    0,    0,
    0,    0,  100,    0,    4,    0,    5,    6,    7,    8,
    9,   10,   11,   12,   13,   14,   15,   16,   17,   18,
   19,   20,    1,    0,    0,    0,    0,    1,    0,    0,
    0,    0,    0,   42,    0,    0,    0,    0,    4,   87,
    0,    0,    0,    4,    0,    0,    0,    0,    0,    0,
    0,    0,   17,   18,   19,   20,    0,   17,   18,   19,
   20,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   25,    0,
    0,   25,    0,    0,   25,   25,   25,   25,   25,   25,
   25,   25,   25,   25,   25,   25,   25,   17,   17,    2,
   17,   30,   12,   17,    0,    2,    0,   30,
};
static const short cue_scanner_check[] = {              260,
    0,   54,  264,  263,  262,  259,   54,   23,  263,   62,
  256,   64,   68,  257,   62,   68,   64,  268,    0,    0,
   68,    0,   78,    0,   77,   78,  264,   43,  257,   77,
   78,  257,  257,  257,   90,  257,  264,   90,   91,  257,
  257,  257,   90,   91,  257,  259,  257,  262,  257,  105,
  267,  274,  105,  270,  262,  272,  273,  105,  275,  276,
  277,  278,  279,  280,  281,  282,  283,  284,  285,  286,
  287,  288,  289,  290,  257,  262,  261,  263,  257,  257,
  268,  263,  265,  266,  259,  257,  263,  257,  256,   22,
  273,  257,  275,  276,  277,  278,  279,  280,  281,  282,
  283,  284,  285,  286,  287,  288,  289,  290,  257,  274,
   62,   64,   78,   91,  105,   83,   -1,   -1,   -1,   -1,
   -1,   -1,  271,   -1,  273,   -1,  275,  276,  277,  278,
  279,  280,  281,  282,  283,  284,  285,  286,  287,  288,
  289,  290,  257,   -1,   -1,   -1,   -1,  257,   -1,   -1,
   -1,   -1,   -1,  268,   -1,   -1,   -1,   -1,  273,  269,
   -1,   -1,   -1,  273,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  287,  288,  289,  290,   -1,  287,  288,  289,
  290,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  268,   -1,
   -1,  271,   -1,   -1,  274,  275,  276,  277,  278,  279,
  280,  281,  282,  283,  284,  285,  286,  268,  269,  268,
  271,  268,  274,  274,   -1,  274,   -1,  274,
};
#define YYFINAL 21
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 290
#define YYTRANSLATE(a) ((a) > YYMAXTOKEN ? (YYMAXTOKEN + 1) : (a))
#if YYDEBUG
static const char *yyname[] = {

"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"CUE_END_OF_LINE",
"CUE_SEMI_COLON","CUE_2DIGIT","CUE_CATALOG_NUMBER","CUE_ISRC_CODE",
"CUE_KEYWORD_STRING","CUE_MSF","CUE_STRING","CUE_CATALOG","CUE_CDTEXTFILE",
"CUE_FLAGS","CUE_FILE","CUE_INDEX","CUE_ISRC","CUE_POSTGAP","CUE_PREGAP",
"CUE_REMARK","CUE_TRACK","CUE_CDTEXT_ARRANGER","CUE_CDTEXT_COMPOSER",
"CUE_CDTEXT_DISC_ID","CUE_CDTEXT_GENRE","CUE_CDTEXT_MESSAGE",
"CUE_CDTEXT_PERFORMER","CUE_CDTEXT_SIZE_INFO","CUE_CDTEXT_SONGWRITER",
"CUE_CDTEXT_TITLE","CUE_CDTEXT_TOC_INFO1","CUE_CDTEXT_TOC_INFO2",
"CUE_CDTEXT_UPC_EAN","CUE_REMARK_LEAD_OUT","CUE_REMARK_ORIGINAL_MEDIA_TYPE",
"CUE_REMARK_RUN_OUT","CUE_REMARK_SESSION","illegal-symbol",
};
static const char *yyrule[] = {
"$accept : cue_main",
"cue_main : cue_main_items cue_file cue_main_items cue_main_tracks",
"cue_main_items :",
"cue_main_items : cue_main_item cue_main_items",
"cue_main_item : cue_catalog",
"cue_main_item : cue_cdtextfile",
"cue_main_item : cue_cdtext",
"cue_main_item : cue_remark_item",
"cue_main_item : cue_empty_line",
"cue_main_tracks :",
"cue_main_tracks : cue_main_track cue_main_tracks",
"cue_main_track : cue_main_track_preceding_items cue_track cue_main_track_leading_items cue_main_track_index_items cue_main_track_trailing_items",
"cue_main_track_preceding_items :",
"cue_main_track_preceding_items : cue_main_track_preceding_item cue_main_track_preceding_items",
"cue_main_track_preceding_item : cue_file",
"cue_main_track_preceding_item : cue_remark_item",
"cue_main_track_preceding_item : cue_empty_line",
"cue_main_track_leading_items :",
"cue_main_track_leading_items : cue_main_track_leading_item cue_main_track_leading_items",
"cue_main_track_leading_item : cue_cdtext",
"cue_main_track_leading_item : cue_flags",
"cue_main_track_leading_item : cue_isrc",
"cue_main_track_leading_item : cue_pregap",
"cue_main_track_leading_item : cue_remark_item",
"cue_main_track_leading_item : cue_empty_line",
"cue_main_track_index_items :",
"cue_main_track_index_items : cue_main_track_index_item cue_main_track_index_items",
"cue_main_track_index_item : cue_index",
"cue_main_track_index_item : cue_remark_item",
"cue_main_track_index_item : cue_empty_line",
"cue_main_track_trailing_items :",
"cue_main_track_trailing_items : cue_main_track_trailing_item cue_main_track_trailing_items",
"cue_main_track_trailing_item : cue_cdtext",
"cue_main_track_trailing_item : cue_postgap",
"cue_main_track_trailing_item : cue_remark_item",
"cue_main_track_trailing_item : cue_empty_line",
"cue_catalog : CUE_CATALOG CUE_CATALOG_NUMBER CUE_END_OF_LINE",
"cue_cdtext : cue_cdtext_type CUE_STRING CUE_END_OF_LINE",
"cue_cdtext_type : CUE_CDTEXT_ARRANGER",
"cue_cdtext_type : CUE_CDTEXT_COMPOSER",
"cue_cdtext_type : CUE_CDTEXT_DISC_ID",
"cue_cdtext_type : CUE_CDTEXT_GENRE",
"cue_cdtext_type : CUE_CDTEXT_MESSAGE",
"cue_cdtext_type : CUE_CDTEXT_PERFORMER",
"cue_cdtext_type : CUE_CDTEXT_SIZE_INFO",
"cue_cdtext_type : CUE_CDTEXT_SONGWRITER",
"cue_cdtext_type : CUE_CDTEXT_TITLE",
"cue_cdtext_type : CUE_CDTEXT_TOC_INFO1",
"cue_cdtext_type : CUE_CDTEXT_TOC_INFO2",
"cue_cdtext_type : CUE_CDTEXT_UPC_EAN",
"cue_cdtextfile : CUE_CDTEXTFILE CUE_STRING CUE_END_OF_LINE",
"cue_file : CUE_FILE CUE_STRING CUE_KEYWORD_STRING CUE_END_OF_LINE",
"cue_flags : CUE_FLAGS cue_flags_types CUE_END_OF_LINE",
"cue_flags_types :",
"cue_flags_types : CUE_KEYWORD_STRING cue_flags_types",
"cue_index : CUE_INDEX CUE_2DIGIT CUE_MSF CUE_END_OF_LINE",
"cue_isrc : CUE_ISRC CUE_ISRC_CODE CUE_END_OF_LINE",
"cue_postgap : CUE_POSTGAP CUE_MSF CUE_END_OF_LINE",
"cue_pregap : CUE_PREGAP CUE_MSF CUE_END_OF_LINE",
"cue_remark_item : cue_lead_out",
"cue_remark_item : cue_original_media_type",
"cue_remark_item : cue_remark",
"cue_remark_item : cue_run_out",
"cue_remark_item : cue_session",
"cue_lead_out : CUE_REMARK_LEAD_OUT CUE_MSF CUE_END_OF_LINE",
"cue_original_media_type : CUE_REMARK_ORIGINAL_MEDIA_TYPE CUE_KEYWORD_STRING CUE_END_OF_LINE",
"cue_remark : cue_remark_start error CUE_END_OF_LINE",
"cue_remark_start : CUE_REMARK",
"cue_run_out : CUE_REMARK_RUN_OUT CUE_MSF CUE_END_OF_LINE",
"cue_session : CUE_REMARK_SESSION CUE_2DIGIT CUE_END_OF_LINE",
"cue_track : CUE_TRACK CUE_2DIGIT CUE_KEYWORD_STRING CUE_END_OF_LINE",
"cue_empty_line : CUE_END_OF_LINE",

};
#endif

int      yydebug;
int      yynerrs;

int      yyerrflag;
int      yychar;
YYSTYPE  yyval;
YYSTYPE  yylval;

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH  10000
#endif
#endif

#define YYINITSTACKSIZE 200

typedef struct {
    unsigned stacksize;
    short    *s_base;
    short    *s_mark;
    short    *s_last;
    YYSTYPE  *l_base;
    YYSTYPE  *l_mark;
} YYSTACKDATA;
/* variables for the parser stack */
static YYSTACKDATA yystack;

int cue_parser_parse_buffer(
     libodraw_handle_t *handle,
     const uint8_t *buffer,
     size_t buffer_size,
     libcerror_error_t **error )
{
	cue_parser_state_t parser_state;
	
	YY_BUFFER_STATE buffer_state = NULL;
	size_t buffer_offset         = 0;
	int result                   = -1;

	if( buffer_size >= 3 )
	{
		if( ( buffer[ 0 ] == 0x0ef )
		 && ( buffer[ 1 ] == 0x0bb )
		 && ( buffer[ 2 ] == 0x0bf ) )
		{
			buffer_offset = 3;
		}
	}
	buffer_state = cue_scanner__scan_buffer(
	                (char *) &( buffer[ buffer_offset ] ),
	                buffer_size - buffer_offset );

	cue_scanner_buffer_offset = (size_t) buffer_offset;

	if( buffer_state != NULL )
	{
		parser_state.handle                         = handle;
		parser_state.error                          = error;
		parser_state.previous_file_index            = -1;
		parser_state.current_file_index             = -1;
		parser_state.file_type                      = 0;
		parser_state.previous_file_sector           = 0;
		parser_state.file_sector                    = 0;
		parser_state.previous_session_start_sector  = 0;
		parser_state.previous_lead_out_start_sector = 0;
		parser_state.previous_track_start_sector    = 0;
		parser_state.current_start_sector           = 0;
		parser_state.session_number_of_sectors      = 0;
		parser_state.lead_out_number_of_sectors     = 0;
		parser_state.track_number_of_sectors        = 0;
		parser_state.previous_track_type            = 0;
		parser_state.current_track_type             = 0;
		parser_state.previous_session               = 0;
		parser_state.current_session                = 0;
		parser_state.previous_lead_out              = 0;
		parser_state.current_lead_out               = 0;
		parser_state.previous_track                 = 0;
		parser_state.current_track                  = 0;
		parser_state.previous_index                 = 0;
		parser_state.current_index                  = 0;

		if( cue_scanner_parse(
		     &parser_state ) == 0 )
		{
			result = 1;
		}
		cue_scanner__delete_buffer(
		 buffer_state );

		if( parser_state.current_session > 0 )
		{
			if( libodraw_handle_append_session(
			     parser_state.handle,
			     parser_state.previous_session_start_sector,
			     0,
			     parser_state.error ) != 1 )
			{
				libcerror_error_set(
				 parser_state.error,
				 LIBCERROR_ERROR_DOMAIN_RUNTIME,
				 LIBCERROR_RUNTIME_ERROR_APPEND_FAILED,
				 "%s: unable to append session.",
				 cue_parser_function );

				result = -1;
			}
		}
		if( parser_state.current_lead_out > parser_state.previous_lead_out )
		{
			if( libodraw_handle_append_lead_out(
			     parser_state.handle,
			     parser_state.previous_lead_out_start_sector,
			     0,
			     parser_state.error ) != 1 )
			{
				libcerror_error_set(
				 parser_state.error,
				 LIBCERROR_ERROR_DOMAIN_RUNTIME,
				 LIBCERROR_RUNTIME_ERROR_APPEND_FAILED,
				 "%s: unable to append lead-out.",
				 cue_parser_function );

				result = -1;
			}
		}
		if( parser_state.current_track > 0 )
		{
			if( parser_state.previous_file_index < 0 )
			{
				parser_state.file_sector = parser_state.previous_track_start_sector;

				parser_state.previous_file_index += 1;
			}
			parser_state.previous_file_sector = parser_state.previous_track_start_sector
			                                  - parser_state.file_sector;

			if( libodraw_handle_append_track(
			     parser_state.handle,
			     parser_state.previous_track_start_sector,
			     0,
			     parser_state.current_track_type,
			     parser_state.current_file_index,
			     parser_state.previous_file_sector,
			     parser_state.error ) != 1 )
			{
				libcerror_error_set(
				 parser_state.error,
				 LIBCERROR_ERROR_DOMAIN_RUNTIME,
				 LIBCERROR_RUNTIME_ERROR_APPEND_FAILED,
				 "%s: unable to append track.",
				 cue_parser_function );

				result = -1;
			}
		}
	}
	cue_scanner_lex_destroy();

	return( result );
}


#if YYDEBUG
#include <stdio.h>		/* needed for printf */
#endif

#include <stdlib.h>	/* needed for malloc, etc */
#include <string.h>	/* needed for memset */

/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(YYSTACKDATA *data)
{
    int i;
    unsigned newsize;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = data->stacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = (int) (data->s_mark - data->s_base);
    newss = (short *)realloc(data->s_base, newsize * sizeof(*newss));
    if (newss == 0)
        return -1;

    data->s_base = newss;
    data->s_mark = newss + i;

    newvs = (YYSTYPE *)realloc(data->l_base, newsize * sizeof(*newvs));
    if (newvs == 0)
        return -1;

    data->l_base = newvs;
    data->l_mark = newvs + i;

    data->stacksize = newsize;
    data->s_last = data->s_base + newsize - 1;
    return 0;
}

#if YYPURE || defined(YY_NO_LEAKS)
static void yyfreestack(YYSTACKDATA *data)
{
    free(data->s_base);
    free(data->l_base);
    memset(data, 0, sizeof(*data));
}
#else
#define yyfreestack(data) /* nothing */
#endif

#define YYABORT  goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR  goto yyerrlab

int
YYPARSE_DECL()
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;
    yystate = 0;

#if YYPURE
    memset(&yystack, 0, sizeof(yystack));
#endif

    if (yystack.s_base == NULL && yygrowstack(&yystack)) goto yyoverflow;
    yystack.s_mark = yystack.s_base;
    yystack.l_mark = yystack.l_base;
    yystate = 0;
    *yystack.s_mark = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = YYLEX) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = yyname[YYTRANSLATE(yychar)];
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack))
        {
            goto yyoverflow;
        }
        yystate = yytable[yyn];
        *++yystack.s_mark = yytable[yyn];
        *++yystack.l_mark = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;

    yyerror(parser_state, "syntax error");

    goto yyerrlab;

yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yystack.s_mark]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yystack.s_mark, yytable[yyn]);
#endif
                if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack))
                {
                    goto yyoverflow;
                }
                yystate = yytable[yyn];
                *++yystack.s_mark = yytable[yyn];
                *++yystack.l_mark = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yystack.s_mark);
#endif
                if (yystack.s_mark <= yystack.s_base) goto yyabort;
                --yystack.s_mark;
                --yystack.l_mark;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = yyname[YYTRANSLATE(yychar)];
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yystack.l_mark[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 36:
	{
		cue_parser_rule_print(
		 "cue_catalog" );
	}
break;
case 37:
	{
		cue_parser_rule_print(
		 "cue_cdtext" );
	}
break;
case 50:
	{
		cue_parser_rule_print(
		 "cue_cdtextfile" );
	}
break;
case 51:
	{
		cue_parser_rule_print(
		 "cue_file" );

		if( yystack.l_mark[-2].string_value.data == NULL )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid filename.",
			 cue_parser_function );

			YYABORT;
		}
/* TODO what about the string quotation marks */
		if( yystack.l_mark[-1].string_value.data == NULL )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid file type.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->file_type = LIBODRAW_FILE_TYPE_UNKNOWN;

		if( yystack.l_mark[-1].string_value.length == 3 )
		{
			if( narrow_string_compare(
			     yystack.l_mark[-1].string_value.data,
			     "MP3",
			     3 ) == 0 )
			{
				( (cue_parser_state_t *) parser_state )->file_type = LIBODRAW_FILE_TYPE_AUDIO_MPEG1_LAYER3;
			}
		}
		else if( yystack.l_mark[-1].string_value.length == 4 )
		{
			if( narrow_string_compare(
			     yystack.l_mark[-1].string_value.data,
			     "AIFF",
			     4 ) == 0 )
			{
				( (cue_parser_state_t *) parser_state )->file_type = LIBODRAW_FILE_TYPE_AUDIO_AIFF;
			}
			else if( narrow_string_compare(
			          yystack.l_mark[-1].string_value.data,
			          "WAVE",
			          4 ) == 0 )
			{
				( (cue_parser_state_t *) parser_state )->file_type = LIBODRAW_FILE_TYPE_AUDIO_WAVE;
			}
		}
		else if( yystack.l_mark[-1].string_value.length == 6 )
		{
			if( narrow_string_compare(
			     yystack.l_mark[-1].string_value.data,
			     "BINARY",
			     6 ) == 0 )
			{
				( (cue_parser_state_t *) parser_state )->file_type = LIBODRAW_FILE_TYPE_BINARY_LITTLE_ENDIAN;
			}
		}
		else if( yystack.l_mark[-1].string_value.length == 8 )
		{
			if( narrow_string_compare(
			     yystack.l_mark[-1].string_value.data,
			     "MOTOROLA",
			     8 ) == 0 )
			{
				( (cue_parser_state_t *) parser_state )->file_type = LIBODRAW_FILE_TYPE_BINARY_BIG_ENDIAN;
			}
		}
		if( libodraw_handle_append_data_file(
		     ( (cue_parser_state_t *) parser_state )->handle,
		     yystack.l_mark[-2].string_value.data,
		     yystack.l_mark[-2].string_value.length,
		     ( (cue_parser_state_t *) parser_state )->file_type,
		     ( (cue_parser_state_t *) parser_state )->error ) != 1 )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_RUNTIME,
			 LIBCERROR_RUNTIME_ERROR_APPEND_FAILED,
			 "%s: unable to append data file.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->current_file_index += 1;
	}
break;
case 52:
	{
		cue_parser_rule_print(
		 "cue_flags" );
	}
break;
case 55:
	{
		cue_parser_rule_print(
		 "cue_index" );

		if( yystack.l_mark[-2].string_value.data == NULL )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid index number.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->previous_index = ( (cue_parser_state_t *) parser_state )->current_index;

		if( ( ( yystack.l_mark[-2].string_value.data )[ 0 ] < '0' )
		 || ( ( yystack.l_mark[-2].string_value.data )[ 0 ] > '9' ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported index number.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->current_index = ( yystack.l_mark[-2].string_value.data )[ 0 ] - '0';

		if( yystack.l_mark[-2].string_value.length == 2 )
		{
			( (cue_parser_state_t *) parser_state )->current_index *= 10;

			if( ( ( yystack.l_mark[-2].string_value.data )[ 1 ] < '0' )
			 || ( ( yystack.l_mark[-2].string_value.data )[ 1 ] > '9' ) )
			{
				libcerror_error_set(
				 ( (cue_parser_state_t *) parser_state )->error,
				 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
				 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
				 "%s: unsupported index number.",
				 cue_parser_function );

				YYABORT;
			}
			( (cue_parser_state_t *) parser_state )->current_index += ( yystack.l_mark[-2].string_value.data )[ 1 ] - '0';
		}
		if( ( ( (cue_parser_state_t *) parser_state )->current_index != 0 )
		 && ( ( (cue_parser_state_t *) parser_state )->current_index != ( ( (cue_parser_state_t *) parser_state )->previous_index + 1 ) ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_RUNTIME,
			 LIBCERROR_RUNTIME_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported index number - values are not sequential.",
			 cue_parser_function );

			YYABORT;
		}
		if( ( yystack.l_mark[-1].string_value.data == NULL )
		 || ( yystack.l_mark[-1].string_value.length != 8 ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid index MSF.",
			 cue_parser_function );

			YYABORT;
		}
		if( ( ( yystack.l_mark[-1].string_value.data )[ 0 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 0 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 1 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 1 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 2 ] != ':' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 3 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 3 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 4 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 4 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 5 ] != ':' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 6 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 6 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 7 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 7 ] > '9' ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported index MSF.",
			 cue_parser_function );

			YYABORT;
		}
		cue_parser_copy_relative_msf_to_lba(
		 yystack.l_mark[-1].string_value.data,
		 ( (cue_parser_state_t *) parser_state )->current_start_sector );

		/* The MSF can be relative to the start of the file
		 */
		if( ( (cue_parser_state_t *) parser_state )->current_start_sector != 0 )
		{
			if( ( ( (cue_parser_state_t *) parser_state )->current_index == 0 )
			 || ( ( (cue_parser_state_t *) parser_state )->current_index == 1 ) )
			{
				if( ( ( (cue_parser_state_t *) parser_state )->session_number_of_sectors == 0 )
				 || ( ( (cue_parser_state_t *) parser_state )->previous_track_type == LIBODRAW_TRACK_TYPE_AUDIO ) )
				{
					if( ( (cue_parser_state_t *) parser_state )->current_start_sector < ( (cue_parser_state_t *) parser_state )->previous_session_start_sector )
					{
						libcerror_error_set(
						 ( (cue_parser_state_t *) parser_state )->error,
						 LIBCERROR_ERROR_DOMAIN_RUNTIME,
						 LIBCERROR_RUNTIME_ERROR_VALUE_OUT_OF_BOUNDS,
						 "%s: invalid session start sector value preceeds previous.",
						 cue_parser_function );

						YYABORT;
					}
					( (cue_parser_state_t *) parser_state )->session_number_of_sectors = ( (cue_parser_state_t *) parser_state )->current_start_sector
					                                                                   - ( (cue_parser_state_t *) parser_state )->previous_session_start_sector;
				}
				if( ( ( (cue_parser_state_t *) parser_state )->lead_out_number_of_sectors == 0 )
				 || ( ( (cue_parser_state_t *) parser_state )->previous_track_type == LIBODRAW_TRACK_TYPE_AUDIO ) )
				{
					if( ( (cue_parser_state_t *) parser_state )->current_start_sector < ( (cue_parser_state_t *) parser_state )->previous_lead_out_start_sector )
					{
						libcerror_error_set(
						 ( (cue_parser_state_t *) parser_state )->error,
						 LIBCERROR_ERROR_DOMAIN_RUNTIME,
						 LIBCERROR_RUNTIME_ERROR_VALUE_OUT_OF_BOUNDS,
						 "%s: invalid lead-out start sector value preceeds previous.",
						 cue_parser_function );

						YYABORT;
					}
					( (cue_parser_state_t *) parser_state )->lead_out_number_of_sectors = ( (cue_parser_state_t *) parser_state )->current_start_sector
					                                                                    - ( (cue_parser_state_t *) parser_state )->previous_lead_out_start_sector;
				}
			}
			if( ( (cue_parser_state_t *) parser_state )->current_index == 1 )
			{
				if( ( (cue_parser_state_t *) parser_state )->track_number_of_sectors == 0 )
				{
					if( ( (cue_parser_state_t *) parser_state )->current_start_sector < ( (cue_parser_state_t *) parser_state )->previous_track_start_sector )
					{
						libcerror_error_set(
						 ( (cue_parser_state_t *) parser_state )->error,
						 LIBCERROR_ERROR_DOMAIN_RUNTIME,
						 LIBCERROR_RUNTIME_ERROR_VALUE_OUT_OF_BOUNDS,
						 "%s: invalid track start sector value preceeds previous.",
						 cue_parser_function );

						YYABORT;
					}
					( (cue_parser_state_t *) parser_state )->track_number_of_sectors = ( (cue_parser_state_t *) parser_state )->current_start_sector
					                                                                 - ( (cue_parser_state_t *) parser_state )->previous_track_start_sector;
				}
			}
		}
		if( ( (cue_parser_state_t *) parser_state )->current_index == 1 )
		{
			if( ( (cue_parser_state_t *) parser_state )->current_session > 1 )
			{
				if( libodraw_handle_append_session(
				     ( (cue_parser_state_t *) parser_state )->handle,
				     ( (cue_parser_state_t *) parser_state )->previous_session_start_sector,
				     ( (cue_parser_state_t *) parser_state )->session_number_of_sectors,
				     ( (cue_parser_state_t *) parser_state )->error ) != 1 )
				{
					libcerror_error_set(
					 ( (cue_parser_state_t *) parser_state )->error,
					 LIBCERROR_ERROR_DOMAIN_RUNTIME,
					 LIBCERROR_RUNTIME_ERROR_APPEND_FAILED,
					 "%s: unable to append session.",
					 cue_parser_function );

					YYABORT;
				}
				( (cue_parser_state_t *) parser_state )->previous_session_start_sector = ( (cue_parser_state_t *) parser_state )->current_start_sector;
				( (cue_parser_state_t *) parser_state )->session_number_of_sectors     = 0;
			}
			if( ( (cue_parser_state_t *) parser_state )->current_lead_out > ( (cue_parser_state_t *) parser_state )->previous_lead_out )
			{
				if( libodraw_handle_append_lead_out(
				     ( (cue_parser_state_t *) parser_state )->handle,
				     ( (cue_parser_state_t *) parser_state )->previous_lead_out_start_sector,
				     ( (cue_parser_state_t *) parser_state )->lead_out_number_of_sectors,
				     ( (cue_parser_state_t *) parser_state )->error ) != 1 )
				{
					libcerror_error_set(
					 ( (cue_parser_state_t *) parser_state )->error,
					 LIBCERROR_ERROR_DOMAIN_RUNTIME,
					 LIBCERROR_RUNTIME_ERROR_APPEND_FAILED,
					 "%s: unable to append lead-out.",
					 cue_parser_function );

					YYABORT;
				}
				( (cue_parser_state_t *) parser_state )->previous_lead_out          = ( (cue_parser_state_t *) parser_state )->current_lead_out;
				( (cue_parser_state_t *) parser_state )->lead_out_number_of_sectors = 0;
			}
			if( ( (cue_parser_state_t *) parser_state )->current_track > 1 )
			{
				if( ( (cue_parser_state_t *) parser_state )->previous_file_index < 0 )
				{
					( (cue_parser_state_t *) parser_state )->file_sector = ( (cue_parser_state_t *) parser_state )->previous_track_start_sector;

					( (cue_parser_state_t *) parser_state )->previous_file_index += 1;
				}
				( (cue_parser_state_t *) parser_state )->previous_file_sector = ( (cue_parser_state_t *) parser_state )->previous_track_start_sector
				                                                              - ( (cue_parser_state_t *) parser_state )->file_sector;

				if( libodraw_handle_append_track(
				     ( (cue_parser_state_t *) parser_state )->handle,
				     ( (cue_parser_state_t *) parser_state )->previous_track_start_sector,
				     ( (cue_parser_state_t *) parser_state )->track_number_of_sectors,
				     ( (cue_parser_state_t *) parser_state )->previous_track_type,
				     ( (cue_parser_state_t *) parser_state )->previous_file_index,
				     ( (cue_parser_state_t *) parser_state )->previous_file_sector,
				     ( (cue_parser_state_t *) parser_state )->error ) != 1 )
				{
					libcerror_error_set(
					 ( (cue_parser_state_t *) parser_state )->error,
					 LIBCERROR_ERROR_DOMAIN_RUNTIME,
					 LIBCERROR_RUNTIME_ERROR_APPEND_FAILED,
					 "%s: unable to append track.",
					 cue_parser_function );

					YYABORT;
				}
				if( ( (cue_parser_state_t *) parser_state )->previous_file_index < ( (cue_parser_state_t *) parser_state )->current_file_index )
				{
					( (cue_parser_state_t *) parser_state )->file_sector = ( (cue_parser_state_t *) parser_state )->current_start_sector;

					( (cue_parser_state_t *) parser_state )->previous_file_index += 1;
				}
				( (cue_parser_state_t *) parser_state )->previous_track_start_sector = ( (cue_parser_state_t *) parser_state )->current_start_sector;
				( (cue_parser_state_t *) parser_state )->track_number_of_sectors     = 0;
			}
		}
	}
break;
case 56:
	{
		cue_parser_rule_print(
		 "cue_isrc" );
	}
break;
case 57:
	{
		cue_parser_rule_print(
		 "cue_postgap" );
	}
break;
case 58:
	{
		cue_parser_rule_print(
		 "cue_pregap" );
	}
break;
case 64:
	{
		cue_parser_rule_print(
		 "cue_lead_out" );

		if( ( yystack.l_mark[-1].string_value.data == NULL )
		 || ( yystack.l_mark[-1].string_value.length != 8 ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid index MSF.",
			 cue_parser_function );

			YYABORT;
		}
		if( ( ( yystack.l_mark[-1].string_value.data )[ 0 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 0 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 1 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 1 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 2 ] != ':' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 3 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 3 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 4 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 4 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 5 ] != ':' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 6 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 6 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 7 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 7 ] > '9' ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported index MSF.",
			 cue_parser_function );

			YYABORT;
		}
		cue_parser_copy_relative_msf_to_lba(
		 yystack.l_mark[-1].string_value.data,
		 ( (cue_parser_state_t *) parser_state )->previous_lead_out_start_sector );

		if( ( (cue_parser_state_t *) parser_state )->track_number_of_sectors == 0 )
		{
			if( ( (cue_parser_state_t *) parser_state )->previous_lead_out_start_sector < ( (cue_parser_state_t *) parser_state )->previous_track_start_sector )
			{
				libcerror_error_set(
				 ( (cue_parser_state_t *) parser_state )->error,
				 LIBCERROR_ERROR_DOMAIN_RUNTIME,
				 LIBCERROR_RUNTIME_ERROR_VALUE_OUT_OF_BOUNDS,
				 "%s: invalid lead-out start sector value preceeds previous track start sector.",
				 cue_parser_function );

				YYABORT;
			}
			( (cue_parser_state_t *) parser_state )->track_number_of_sectors = ( (cue_parser_state_t *) parser_state )->previous_lead_out_start_sector
			                                                                 - ( (cue_parser_state_t *) parser_state )->previous_track_start_sector;
		}
		( (cue_parser_state_t *) parser_state )->current_lead_out += 1;
	}
break;
case 65:
	{
		cue_parser_rule_print(
		 "cue_original_media_type" );
	}
break;
case 66:
	{
		cue_parser_rule_print(
		 "cue_remark" );

		cue_scanner_suppress_error = 0;
	}
break;
case 67:
	{
		/* The build-in rule error will gobble up all the tokens until the end-of-line
		 * because these are no syntax errors suppress the error output
		 */
		cue_scanner_suppress_error = 1;
	}
break;
case 68:
	{
		cue_parser_rule_print(
		 "cue_run_out" );

		if( ( yystack.l_mark[-1].string_value.data == NULL )
		 || ( yystack.l_mark[-1].string_value.length != 8 ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid index MSF.",
			 cue_parser_function );

			YYABORT;
		}
		if( ( ( yystack.l_mark[-1].string_value.data )[ 0 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 0 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 1 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 1 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 2 ] != ':' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 3 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 3 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 4 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 4 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 5 ] != ':' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 6 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 6 ] > '9' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 7 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 7 ] > '9' ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported index MSF.",
			 cue_parser_function );

			YYABORT;
		}
		cue_parser_copy_relative_msf_to_lba(
		 yystack.l_mark[-1].string_value.data,
		 ( (cue_parser_state_t *) parser_state )->current_start_sector );

		if( ( (cue_parser_state_t *) parser_state )->track_number_of_sectors == 0 )
		{
			if( ( (cue_parser_state_t *) parser_state )->current_start_sector < ( (cue_parser_state_t *) parser_state )->previous_track_start_sector )
			{
				libcerror_error_set(
				 ( (cue_parser_state_t *) parser_state )->error,
				 LIBCERROR_ERROR_DOMAIN_RUNTIME,
				 LIBCERROR_RUNTIME_ERROR_VALUE_OUT_OF_BOUNDS,
				 "%s: invalid track start sector value preceeds previous.",
				 cue_parser_function );

				YYABORT;
			}
			( (cue_parser_state_t *) parser_state )->track_number_of_sectors = ( (cue_parser_state_t *) parser_state )->current_start_sector
									                 - ( (cue_parser_state_t *) parser_state )->previous_track_start_sector;
		}
	}
break;
case 69:
	{
		cue_parser_rule_print(
		 "cue_session" );

		if( yystack.l_mark[-1].string_value.data == NULL )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid session number.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->previous_session = ( (cue_parser_state_t *) parser_state )->current_session;

		if( ( ( yystack.l_mark[-1].string_value.data )[ 0 ] < '0' )
		 || ( ( yystack.l_mark[-1].string_value.data )[ 0 ] > '9' ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported session number.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->current_session = ( yystack.l_mark[-1].string_value.data )[ 0 ] - '0';

		if( yystack.l_mark[-1].string_value.length == 2 )
		{
			( (cue_parser_state_t *) parser_state )->current_session *= 10;

			if( ( ( yystack.l_mark[-1].string_value.data )[ 1 ] < '0' )
			 || ( ( yystack.l_mark[-1].string_value.data )[ 1 ] > '9' ) )
			{
				libcerror_error_set(
				 ( (cue_parser_state_t *) parser_state )->error,
				 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
				 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
				 "%s: unsupported session number.",
				 cue_parser_function );

				YYABORT;
			}
			( (cue_parser_state_t *) parser_state )->current_session += ( yystack.l_mark[-1].string_value.data )[ 1 ] - '0';
		}
		if( ( ( (cue_parser_state_t *) parser_state )->current_session != 0 )
		 && ( ( (cue_parser_state_t *) parser_state )->current_session != ( ( (cue_parser_state_t *) parser_state )->previous_session + 1 ) ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_RUNTIME,
			 LIBCERROR_RUNTIME_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported session number - values are not sequential.",
			 cue_parser_function );

			YYABORT;
		}
	}
break;
case 70:
	{
		cue_parser_rule_print(
		 "cue_track" );

		if( yystack.l_mark[-2].string_value.data == NULL )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid track number.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->previous_track = ( (cue_parser_state_t *) parser_state )->current_track;

		if( ( ( yystack.l_mark[-2].string_value.data )[ 0 ] < '0' )
		 || ( ( yystack.l_mark[-2].string_value.data )[ 0 ] > '9' ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported track number.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->current_track = ( yystack.l_mark[-2].string_value.data )[ 0 ] - '0';

		if( yystack.l_mark[-2].string_value.length == 2 )
		{
			( (cue_parser_state_t *) parser_state )->current_track *= 10;

			if( ( ( yystack.l_mark[-2].string_value.data )[ 1 ] < '0' )
			 || ( ( yystack.l_mark[-2].string_value.data )[ 1 ] > '9' ) )
			{
				libcerror_error_set(
				 ( (cue_parser_state_t *) parser_state )->error,
				 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
				 LIBCERROR_ARGUMENT_ERROR_UNSUPPORTED_VALUE,
				 "%s: unsupported track number.",
				 cue_parser_function );

				YYABORT;
			}
			( (cue_parser_state_t *) parser_state )->current_track += ( yystack.l_mark[-2].string_value.data )[ 1 ] - '0';
		}
		if( ( ( (cue_parser_state_t *) parser_state )->current_track != 0 )
		 && ( ( (cue_parser_state_t *) parser_state )->current_track != ( ( (cue_parser_state_t *) parser_state )->previous_track + 1 ) ) )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_RUNTIME,
			 LIBCERROR_RUNTIME_ERROR_UNSUPPORTED_VALUE,
			 "%s: unsupported track number - values are not sequential.",
			 cue_parser_function );

			YYABORT;
		}
		if( yystack.l_mark[-1].string_value.data == NULL )
		{
			libcerror_error_set(
			 ( (cue_parser_state_t *) parser_state )->error,
			 LIBCERROR_ERROR_DOMAIN_ARGUMENTS,
			 LIBCERROR_ARGUMENT_ERROR_INVALID_VALUE,
			 "%s: invalid track number.",
			 cue_parser_function );

			YYABORT;
		}
		( (cue_parser_state_t *) parser_state )->previous_track_type = ( (cue_parser_state_t *) parser_state )->current_track_type;

		( (cue_parser_state_t *) parser_state )->previous_index     = 0;
		( (cue_parser_state_t *) parser_state )->current_index      = 0;
		( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_UNKNOWN;

		if( yystack.l_mark[-1].string_value.length == 3 )
		{
			if( narrow_string_compare(
			     yystack.l_mark[-1].string_value.data,
			     "CDG",
			     3 ) == 0 )
			{
				( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_CDG;
			}
		}
		else if( yystack.l_mark[-1].string_value.length == 5 )
		{
			if( narrow_string_compare(
			     yystack.l_mark[-1].string_value.data,
			     "AUDIO",
			     5 ) == 0 )
			{
				( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_AUDIO;
			}
		}
		else if( yystack.l_mark[-1].string_value.length == 8 )
		{
			if( narrow_string_compare(
			     yystack.l_mark[-1].string_value.data,
			     "CDI",
			     3 ) == 0 )
			{
				if( ( yystack.l_mark[-1].string_value.data )[ 3 ] == '/' )
				{
					if( narrow_string_compare(
					     &( ( yystack.l_mark[-1].string_value.data )[ 4 ] ),
					     "2336",
					     4 ) == 0 )
					{
						 ( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_CDI_2336;
					}
					else if( narrow_string_compare(
						  &( ( yystack.l_mark[-1].string_value.data )[ 4 ] ),
						  "2352",
						  4 ) == 0 )
					{
						 ( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_CDI_2352;
					}
				}
			}
		}
		else if( yystack.l_mark[-1].string_value.length == 10 )
		{
			if( narrow_string_compare(
			     yystack.l_mark[-1].string_value.data,
			     "MODE",
			     4 ) == 0 )
			{
				if( ( yystack.l_mark[-1].string_value.data )[ 5 ] == '/' )
				{
					if( ( yystack.l_mark[-1].string_value.data )[ 4 ] == '1' )
					{
						if( narrow_string_compare(
						     &( ( yystack.l_mark[-1].string_value.data )[ 6 ] ),
						     "2048",
						     4 ) == 0 )
						{
							 ( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_MODE1_2048;
						}
						else if( narrow_string_compare(
						          &( ( yystack.l_mark[-1].string_value.data )[ 6 ] ),
						          "2352",
						          4 ) == 0 )
						{
							 ( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_MODE1_2352;
						}
					}
					else if( ( yystack.l_mark[-1].string_value.data )[ 4 ] == '2' )
					{
						if( narrow_string_compare(
						     &( ( yystack.l_mark[-1].string_value.data )[ 6 ] ),
						     "2048",
						     4 ) == 0 )
						{
							 ( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_MODE2_2048;
						}
						else if( narrow_string_compare(
						          &( ( yystack.l_mark[-1].string_value.data )[ 6 ] ),
						          "2324",
						          4 ) == 0 )
						{
							 ( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_MODE2_2324;
						}
						else if( narrow_string_compare(
						          &( ( yystack.l_mark[-1].string_value.data )[ 6 ] ),
						          "2336",
						          4 ) == 0 )
						{
							 ( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_MODE2_2336;
						}
						else if( narrow_string_compare(
						          &( ( yystack.l_mark[-1].string_value.data )[ 6 ] ),
						          "2352",
						          4 ) == 0 )
						{
							 ( (cue_parser_state_t *) parser_state )->current_track_type = LIBODRAW_TRACK_TYPE_MODE2_2352;
						}
					}
				}
			}
		}
	}
break;
    }
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yystack.s_mark = YYFINAL;
        *++yystack.l_mark = yyval;
        if (yychar < 0)
        {
            if ((yychar = YYLEX) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = yyname[YYTRANSLATE(yychar)];
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yystack.s_mark, yystate);
#endif
    if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack))
    {
        goto yyoverflow;
    }
    *++yystack.s_mark = (short) yystate;
    *++yystack.l_mark = yyval;
    goto yyloop;

yyoverflow:
    yyerror(parser_state, "yacc stack overflow");

yyabort:
    yyfreestack(&yystack);
    return (1);

yyaccept:
    yyfreestack(&yystack);
    return (0);
}
